locals {
  # moderation.apis.holi.social
  production_backend_zone   = data.terraform_remote_state.holi_infra_state.outputs.dns_apis_zone_name
  production_backend_domain = "moderation.${data.terraform_remote_state.holi_infra_state.outputs.dns_apis_zone_domain}."

  # ENV.moderation.apis.holi.social
  development_backend_zone   = data.terraform_remote_state.holi_infra_state.outputs.dns_apis_zone_name
  development_backend_domain = "${local.environment_name}.${local.production_backend_domain}"

  backend_domain      = local.environment == "production" ? local.production_backend_domain : local.development_backend_domain
  backend_domain_zone = local.environment == "production" ? local.production_backend_zone : local.development_backend_zone
  backend_name        = format(local.name_template, "backend")
}

resource "google_sql_user" "backend" {
  project  = data.terraform_remote_state.holi_infra_state.outputs.shared_project_id
  instance = local.shared_db_instance

  name     = local.backend_name
  password = data.google_secret_manager_secret_version_access.moderation_db_password.secret_data
}

resource "google_sql_database" "backend" {
  project  = data.terraform_remote_state.holi_infra_state.outputs.shared_project_id
  instance = local.shared_db_instance
  name     = local.backend_name

  # on destroy, the user needs to be kept until the db is destroyed
  depends_on = [google_sql_user.backend, google_sql_user.federator_user]
}

resource "google_cloud_run_service" "moderation_backend" {
  project  = data.terraform_remote_state.holi_infra_state.outputs.shared_project_id
  name     = local.backend_name
  location = local.default_region

  depends_on = [google_service_account.moderation, google_project_iam_member.moderation]

  autogenerate_revision_name = true

  template {
    spec {
      service_account_name = google_service_account.moderation.email

      containers {
        image = "${data.terraform_remote_state.holi_infra_state.outputs.artifact_registry_location}/holi-moderation-backend:${var.backend_image_tag}"
        ports {
          container_port = 3000
        }

        env {
          name  = "LISTEN_ADDR"
          value = "0.0.0.0"
        }

        env {
          name  = "NODE_ENV"
          value = "production"
        }

        env {
          name  = "OKUNA_DB_HOST"
          value = local.shared_db_host
        }

        env {
          name = "OKUNA_DB_USER"
          value_from {
            secret_key_ref {
              key  = "latest"
              name = data.google_secret_manager_secret_version_access.okuna_db_username.secret
            }
          }
        }

        env {
          name = "OKUNA_DB_PASSWORD"
          value_from {
            secret_key_ref {
              key  = "latest"
              name = data.google_secret_manager_secret_version_access.okuna_db_password.secret
            }
          }
        }

        env {
          name  = "OKUNA_ADMIN_BASE_URL"
          value = local.environment == "production" ? "https://0kuna-adm1n.holi.social/production/admin" : "https://0kuna-adm1n.holi.social/staging/admin"
        }

        env {
          name  = "MODERATION_DB_HOST"
          value = local.shared_db_host
        }

        env {
          name  = "MODERATION_DB_USER"
          value = google_sql_user.backend.name
        }

        env {
          name = "MODERATION_DB_PASSWORD"
          value_from {
            secret_key_ref {
              key  = "latest"
              name = data.google_secret_manager_secret_version_access.moderation_db_password.secret
            }
          }
        }

        env {
          name  = "MODERATION_DB_NAME"
          value = google_sql_database.backend.name
        }

        env {
          name = "MODERATION_JWT_SECRET"
          value_from {
            secret_key_ref {
              key  = "latest"
              name = data.google_secret_manager_secret_version_access.jwt_secret.secret
            }
          }
        }

        env {
          name  = "PUBSUB_TOPIC_NAME"
          value = google_pubsub_topic.moderation.name
        }

        env {
          name = "INITIAL_USER_PASSWORD"
          value_from {
            secret_key_ref {
              name = "MODERATION_INITIAL_USER_PASSWORD"
              key  = "latest"
            }
          }
        }

        resources {
          limits = {
            cpu    = local.environment == "production" ? "1000m" : "1000m"
            memory = local.environment == "production" ? "512Mi" : "512Mi"
          }
          requests = {
            cpu    = local.environment == "production" ? "1000m" : "1000m"
            memory = local.environment == "production" ? "512Mi" : "512Mi"
          }
        }

        startup_probe {
          timeout_seconds   = 5
          period_seconds    = 5
          failure_threshold = 6

          http_get {
            path = "/health"
          }
        }

        liveness_probe {
          timeout_seconds   = 5
          period_seconds    = 30
          failure_threshold = 5

          http_get {
            path = "/health"
          }
        }
      }

      container_concurrency = 80
      timeout_seconds       = 300
    }

    metadata {
      annotations = {
        "autoscaling.knative.dev/minScale" = local.environment == "production" ? "1" : "0"
        "autoscaling.knative.dev/maxScale" = "1" //before scaling up, make sure to check how this affects the Cron job

        "run.googleapis.com/vpc-access-connector" = data.terraform_remote_state.holi_infra_state.outputs.vpc_access_connector_name
        "run.googleapis.com/vpc-access-egress"    = "private-ranges-only"
        "run.googleapis.com/startup-cpu-boost"    = "true"
      }
      # labels set on the revision level
      labels = {
        "environment_type" = local.environment_type
        "holi_service"     = "moderation-backend"
      }
    }
  }

  metadata {
    annotations = {
      "run.googleapis.com/ingress" = "internal"
    }
    # labels set on the service level
    labels = {
      "environment_type" = local.environment_type
      "holi_service"     = "moderation-backend"
    }
  }

  traffic {
    percent         = 100
    latest_revision = true
  }
}

resource "google_cloud_run_domain_mapping" "backend_domain" {
  project  = google_cloud_run_service.moderation_backend.project
  location = google_cloud_run_service.moderation_backend.location
  name     = trim(local.environment == "production" ? local.production_backend_domain : local.development_backend_domain, ".")

  metadata {
    namespace = google_cloud_run_service.moderation_backend.project
  }

  spec {
    route_name = google_cloud_run_service.moderation_backend.name
  }
}

data "google_iam_policy" "backend" {
  binding {
    role = "roles/run.invoker"
    members = [
      "allUsers",
    ]
  }
}

resource "google_cloud_run_service_iam_policy" "backend" {
  location = google_cloud_run_service.moderation_backend.location
  project  = google_cloud_run_service.moderation_backend.project
  service  = google_cloud_run_service.moderation_backend.name

  policy_data = data.google_iam_policy.backend.policy_data
}
